use gtk4 :: * ;
use gtk4 :: Align :: * ;
use gtk4 :: Orientation :: * ;
use gtk4 :: gdk :: * ;
use gtk4 :: gio :: * ;
use gtk4 :: pango :: * ;
use gtk4 :: pango :: Weight :: * ;
use gtk4 :: prelude :: * ;
use std :: cell :: * ;
use std :: iter :: * ;
use std :: path :: PathBuf ;

use crate :: ui :: collatable_container :: * ;
use crate :: ui :: component :: * ;



pub struct CollatedMediaGrid < A : MediaAdapter > {
	media_widget_pairs : RefCell < Vec < ( A :: Overview , Button ) > > ,
	grid_widget : FlowBox ,
}

impl < A : MediaAdapter > CollatedMediaGrid <A> {
	pub fn new () -> Self {
		let grid_widget = flow_box ! (
			@ orientation : Horizontal ;
			@ homogeneous : true ;
			@ selection_mode : SelectionMode :: None ;
		) ;
		let media_widget_pairs = RefCell :: new ( vec ! () ) ;

		Self { media_widget_pairs , grid_widget }
	}

	pub async fn set_media ( & self , media : Vec < A :: Overview > , sorting : A :: Sorting ) {
		// TODO: Check if we should use `MainContext :: invoke_local` here

		let mut widgets = Vec :: new () ;
		for media in media . as_slice () {
			widgets . push ( create_media_entry (media) . await ) ;
		}
		self . media_widget_pairs . replace ( zip ( media , widgets ) . collect () ) ;

		for ( _ , widget ) in self . sort_media_widget_pairs (sorting) {
			self . grid_widget . append ( & widget ) ;
		}
	}

	pub fn set_sorting ( & self , sorting : A :: Sorting ) {
		self . grid_widget . remove_all () ;

		for ( _ , widget ) in self . sort_media_widget_pairs (sorting) {
			self . grid_widget . append ( & widget ) ;
		}
	}

	fn sort_media_widget_pairs ( & self , sorting : A :: Sorting ) -> Vec < ( A :: Overview , Button ) > {
		let mut sorted = Vec :: from (
			self . media_widget_pairs . borrow () . as_slice () ,
		) ;

		sorted . sort_by ( | ( media_1 , _ ) , ( media_2 , _ ) |
			A :: compare_by ( media_1 , media_2 , sorting ) ,
		) ;

		// See it, say it, ...
		sorted
	}
}

impl < A : MediaAdapter > Component for CollatedMediaGrid <A> {
	fn get_widget ( & self ) -> & FlowBox { & self . grid_widget }
}

async fn create_media_entry < M : MediaOverview > ( media : & M ) -> Button {
	button ! (
		@ css_classes : & [ "flat" , "open-collection-item-button" ] ;
		@ connect_clicked : |_| todo ! () ;
		& g_box ! (
			@ option_children ;
			@ orientation : Vertical ;
			@ margin_top : 20 ;
			@ margin_bottom : 20 ;

			match media . get_poster_file_path () {
				Some (poster_file_path) => {
					let poster_file_path = PathBuf :: from (poster_file_path) ;  // God forbid `Path` would work with `clone ! ()`
					let poster_texture = spawn_blocking ( move ||
						Texture :: from_filename (poster_file_path) . unwrap ()
					) . await . unwrap () ;
					Some ( image ! (
						@ paintable : & poster_texture ;
						@ width_request : 300 ;
						@ height_request : 300 ;
						@ margin_bottom : 10 ;
					) )
				} ,
				None => None ,
			} . as_ref () ,

			Some ( & label ! (
				@ justify : Justification :: Center ;
				@ wrap : true ;
				@ max_width_chars : 1 ;  // Not the actual limit, used instead to wrap more aggressively
				@ attributes : & pango_attributes ! ( @ scale : SCALE_LARGE ; @ weight : Bold ; ) ;
				media . get_name () . as_str () ,
			) ) ,

			match media . get_original_name () {
				Some (original_name) => Some ( label ! (
					@ justify : Justification :: Center ;
					@ wrap : true ;
					@ max_width_chars : 1 ;  // Not the actual limit, used instead to wrap more aggressively
					original_name . as_str () ,
				) ) ,
				None => None ,
			} . as_ref () ,

			Some ( & g_box ! (
				@ option_children ;
				@ orientation : Horizontal ;
				@ halign : Center ;
				@ spacing : 20 ;
				Some ( & label ! ( media . get_release_date () . as_str () ) ) ,
				match media . get_runtime_minutes () {
					Some (runtime_minutes) => Some (
						label ! ( format ! ( "{}m" , runtime_minutes ) . as_str () ) ,
					) ,
					None => None ,
				} . as_ref () ,
			) ) ,
		) ,
	)
}
